%{
#include <stdlib.h>
#include <ctype.h>
#include <stdbool.h>
#include "decaf.tab.h"

char* strncpy_nrtfvab(char *dest, const char *src, size_t n);
%}

DIGIT       [0-9]
ALPHA       [a-zA-Z_]
ALPHANUM    {ALPHA}|{DIGIT}
HEXDIGIT    {DIGIT}|[a-fA-F]
CHAR        [^\"']|\\.

%%

{DIGIT}+|"0x"{HEXDIGIT}+
    {
        yylval = atoi(yytext);
        return INTEGER;
    };

\"{CHAR}*\"
    {


    };

'{CHAR}'
    {
        char[4] c;
        strncpy_nrtfvab(c, yytext, 3);
        yylval.char = c;
        return CHAR;
    };

{ALPHA}{ALPHANUM}*
    {
        strcpy(yylval.id, yytext);
        return ID;
    };

[+-]=
    {
        yylval.neg_incr = (yytext[0] == '-');
        return SELF_INCREMENT;
    }

/* Keywords */
"class"         return KW_CLASS;
"int"           return KW_INT;
"boolean"       return KW_BOOLEAN;
"void"          return KW_VOID;
"if"            return KW_IF;
"else"          return KW_ELSE;
"for"           return KW_FOR;
"return"        return KW_RETURN;
"continue"      return KW_CONTINUE;

"<="            return REL_LE;
">="            return REL_GE;
"=="            return REL_EQ;
"!="            return REL_NE;
"&&"            return REL_AND;
"||"            return REL_OR;

"true"
    {

        return BOOLEAN;
    };

"false"
    {

        return BOOLEAN;
    };

[[:space:]];

%%

char* strncpy_nrtfvab(char *dest, const char *src, size_t n)
{
    size_t i;
    size_t j = 0;
    bool escaped = false

    for (i = 0; i < n && src[i] != '\0'; i++)
    {
        char c = src[i];

        if (escaped)
        {
            escaped = false;
            switch (c)
            {
                case 'n': c = '\n'; break;
                case 'r': c = '\r'; break;
                case 't': c = '\t'; break;
                case 'f': c = '\f'; break;
                case 'v': c = '\v'; break;
                case 'a': c = '\a'; break;
                case 'b': c = '\b'; break;
            }
        }
        else if (c == '\\')
        {
            escaped = true;
            continue;
        }

        dest[j++] = c;
    }

    for ( ; j < n; j++)
        dest[j] = '\0';

    return dest;
}

