%{

%}

%union {
    int integer;
    char* string;
    char character;
    bool boolean;
}

%token <string> ID
%token <integer> INTEGER
%token <boolean> BOOLEAN
%token <string> STRING
%token <character> CHAR

%token REL_LE
%token REL_GE
%token REL_EQ
%token REL_NE
%token REL_AND
%token REL_OR

%token <character> SELF_INCREMENT

%token KW_CLASS
%token KW_INT
%token KW_BOOLEAN
%token KW_VOID
%token KW_IF
%token KW_ELSE
%token KW_FOR
%token KW_RETURN
%token KW_CONTINUE

%start program

%%


program
    : KW_CLASS ID '{' field_decls method_decls '}'
    ;

field_decls
    : field_decls field_decl
    | %empty
    ;

field_decl
    : type field_ids ';'
    ;

field_ids
    : field_id
    | field_ids ',' field_id
    ;

field_id
    : ID
    | ID '[' INTEGER ']'
    ;

type
    : KW_INT
    | KW_BOOLEAN
    ;

method_decls
    : method_decls method_decl
    | %empty
    ;

method_decl
    : method_type ID '(' method_args ')' block

method_type
    : type
    | KW_VOID
    ;

method_args
    : type ID
    | method_args ',' type ID
    ;

statements
    : statements statement
    | %empty
    ;

statement
    : location assign_op expr ';'
    | method_call ';'
    | KW_IF '(' expr ')' block
    | KW_IF '(' expr ')' block KW_ELSE block
    | KW_FOR ID '=' expr ',' expr block
    | KW_RETURN ';'
    | KW_RETURN expr ';'
    | KW_CONTINUE ';'
    | block
    ;

block
    : '{' var_decls statements '}'
    ;

var_decls
    : var_decls var_decl
    | %empty
    ;

var_decl
    : type var_ids ';'
    ;

var_ids
    : ID
    | var_ids ',' ID
    ;

method_call
    : method_name '(' ')'
    | method_name '(' expr_set ')'
    ;

expr_set
    : expr
    | expr_set ',' expr
    ;

// TODO: check utility ?
method_name
    : ID
    ;

location
    : ID
    | ID '[' expr ']'
    ;

expr
    : location
    | method_call
    | literal
    | expr bin_op expr
    | '-' expr
    | '!' expr
    | '(' expr ')'
    ;

assign_op
    : '='
    | SELF_INCREMENT
    ;

bin_op
    : arith_op
    | rel_op
    | eq_op
    | cond_op
    ;

arith_op
    : '+' | '-' | '*' | '/' | '%' ;

rel_op
    : '<'
    | '>'
    | REL_LE
    | REL_GE
    ;

eq_op
    : REL_EQ
    | REL_NE
    ;

cond_op
    : REL_AND
    | REL_OR
    ;

literal
    : INTEGER
    | CHAR
    | STRING
    | BOOLEAN
    ;

%%